@@ -765,12 +765,7 @@ Hi {{NAME-5651}},
 > If you'd like me to stop sending you emails, please click here ( %unsubscribe_url% )
 """
 
-LONG_SAMPLES_ENTITIES_DICT = {
-    "entities": [
-        (1661, 1677, "Product Training"),
-        (1787, 1815, "Action item not completed"),
-    ]
-}
+LONG_SAMPLES_ENTITIES_DICT = {"entities": [(85, 93, "greetings_label")]}
 
 LONG_TRAIN_DATA = [(LONG_SAMPLE, LONG_SAMPLES_ENTITIES_DICT)]
 
